FnParser

Struct FnParser 

pub struct FnParser<I: View<V = Seq<u8>>, O: DeepView, Spec: SpecParser<PVal = O::V>, Exec: Fn(&I) -> PResult<O>> {
    pub exec_fn: Exec,
    pub spec_fn: Ghost<Spec>,
    pub _marker: PhantomData<(I, O)>,
}
Pairs an executable parser closure with a ghost specification parser.

Fields

exec_fn: Execspec_fn: Ghost<Spec>_marker: PhantomData<(I, O)>

Implementations

Source

impl<I, O, Spec, Exec> FnParser<I, O, Spec, Exec>
where I: View<V = Seq<u8>>, O: DeepView, Spec: Productive<PVal = O::V>, Exec: Fn(&I) -> PResult<O>,

pub exec fn new(exec_fn: Exec, Ghost(spec_fn): Ghost<Spec>) -> parser : Self

requires
spec_fn.safe_inv(),
spec_fn.productive_inv(),
forall |i: &I| #[trigger] call_requires(exec_fn, (i,)),
forall |i: &I, r: PResult<O>| {
    #[trigger] call_ensures(exec_fn, (i,), r)
        ==> parse_matches_spec(r, spec_fn.spec_parse(i@))
},
ensures
parser.exec_inv(),
parser.safe_inv(),
parser.productive_inv(),
parser.spec_fn == spec_fn,

Constructs a safe, productive parser callback with a ghost specification.